@@ -4,7 +4,6 @@
  * Sparse matrices in compressed column-oriented form
  */
 #include "Csparse.h"
-#include "Tsparse.h"
 #include "chm_common.h"
 #include "cs_utils.h" /* -> ./cs.h  for cs_dmperm() */
 
@@ -37,11 +36,6 @@ Rboolean isValid_Csparse(SEXP x)
     return TRUE;
 }
 
-SEXP Csparse_validate(SEXP x) {
-    return Csparse_validate_(x, FALSE);
-}
-
-
 #define _t_Csparse_validate
 #include "t_Csparse_validate.c"
 
@@ -60,6 +54,14 @@ SEXP Csparse_sort (SEXP x) {
    return x;
 }
 
+/* MJ: no longer needed ... replacement in ./validate.c */
+#if 0
+
+SEXP Csparse_validate(SEXP x)
+{
+    return Csparse_validate_(x, FALSE);
+}
+
 SEXP Rsparse_validate(SEXP x)
 {
     /* NB: we do *NOT* check a potential 'x' slot here, at all */
@@ -105,6 +107,8 @@ SEXP Rsparse_validate(SEXP x)
     return ScalarLogical(1);
 }
 
+#endif /* MJ */
+
 /* MJ: no longer needed ... prefer R_sparse_as_dense() */
 #if 0
 
@@ -329,6 +333,9 @@ SEXP Csparse_to_vector(SEXP x)
 
 #endif /* MJ */
 
+/* MJ: no longer needed ... prefer CRsparse_as_Tsparse() */
+#if 0
+
 SEXP Csparse_to_Tsparse(SEXP x, SEXP tri)
 {
     CHM_SP chxs = AS_CHM_SP__(x);
@@ -343,6 +350,8 @@ SEXP Csparse_to_Tsparse(SEXP x, SEXP tri)
 			       GET_SLOT(x, Matrix_DimNamesSym));
 }
 
+#endif /* MJ */
+
 /* MJ: unused */
 #if 0
 
@@ -369,7 +378,7 @@ SEXP Csparse_to_tTsparse(SEXP x, SEXP uplo, SEXP diag)
 			       GET_SLOT(x, Matrix_DimNamesSym));
 }
 
-#endif
+#endif /* MJ */
 
 /* MJ: no longer needed ... prefer R_sparse_as_general() */
 #if 0
@@ -429,7 +438,7 @@ SEXP Csparse_general_to_symmetric(SEXP x, SEXP uplo, SEXP sym_dmns)
 #endif /* MJ */
 
 /* MJ: no longer needed ... prefer R_sparse_transpose() */
-/* MJ: however, some reverse dependencies built with Matrix < 1.4-2 need it */
+/* MJ: however, some reverse dependencies built with Matrix < 1.5-0 need it */
 #ifdef Matrix_SupportingCachedMethods
 
 SEXP Csparse_transpose(SEXP x, SEXP tri)
@@ -773,7 +782,7 @@ SEXP Csparse_crossprod(SEXP x, SEXP trans, SEXP triplet, SEXP bool_arith)
 #ifdef AS_CHM_DIAGU2N_FIXED_FINALLY
     CHM_TR cht = tripl ? AS_CHM_TR(x) : (CHM_TR) NULL;  int nprot = 1;
 #else /* workaround needed:*/
-    SEXP xx = PROTECT(Tsparse_diagU2N(x));
+    SEXP xx = PROTECT(R_sparse_diag_U2N(x));
     CHM_TR cht = tripl ? AS_CHM_TR__(xx) : (CHM_TR) NULL; int nprot = 2;
 #endif
     CHM_SP chcp, chxt, chxc,
@@ -917,7 +926,7 @@ SEXP Csparse_vertcat(SEXP x, SEXP y)
 }
 
 /* MJ: no longer needed ... prefer R_sparse_band() */
-/* MJ: however, some reverse dependencies built with Matrix < 1.4-2 need it */
+/* MJ: however, some reverse dependencies built with Matrix < 1.5-0 need it */
 #ifdef Matrix_SupportingCachedMethods
 
 SEXP Csparse_band(SEXP x, SEXP k1, SEXP k2)
@@ -933,6 +942,9 @@ SEXP Csparse_band(SEXP x, SEXP k1, SEXP k2)
 
 #endif /* MJ */
 
+/* MJ: no longer needed ... prefer R_sparse_diag_(U2N|N2U)() */
+#if 0
+
 SEXP Csparse_diagU2N(SEXP x)
 {
     const char *cl = class_P(x);
@@ -984,6 +996,8 @@ SEXP Csparse_diagN2U(SEXP x)
     }
 }
 
+#endif
+
 /**
  * Indexing aka subsetting : Compute  x[i,j], also for vectors i and j
  * Working via CHOLMOD_submatrix, see ./CHOLMOD/MatrixOps/cholmod_submatrix.c
