@@ -28,18 +28,19 @@ stopifnot(validObject(tmm), dim(tmm) == dm[2:1],
           identical(as(tmm, "matrix"), t(as(mm, "matrix"))))
 
 ## from a bug report by Guissepe Ragusa <gragusa@ucsd.edu>
+RNGversion("3.6.0")# future proof
 set.seed(101)
 for(i in 1:10) {
     A <- matrix(rnorm(400), nrow = 100, ncol = 4)
     A[A < +1] <- 0 ; Am <- A
-    Acsc <- as(Am, "dgCMatrix")
-    A    <- as(Am, "dgeMatrix")
+    Acsc <- as(Am, "CsparseMatrix")
+    A    <- as(Am, "unpackedMatrix")
     b <- matrix(rnorm(400), nrow = 4, ncol = 100)
-    B <- as(b, "dgeMatrix")
+    B <- as(b, "unpackedMatrix")
     assert.EQ.mat(A %*% B, Am %*%  b)
     assert.EQ.mat(B %*% A,  b %*% Am)
-    stopifnot(identical(A, as(Acsc, "dgeMatrix")),
-              identical(Acsc, as(A, "dgCMatrix")),
+    stopifnot(identical(A, as(Acsc, "unpackedMatrix")),
+              identical(Acsc, as(A, "CsparseMatrix")),
               is.all.equal4(A %*% B, Acsc %*% B,
                             A %*% b, Acsc %*% b),
               is.all.equal4(b %*% A, b %*% Acsc,
@@ -53,25 +54,25 @@ for(i in 1:10) {
 N <- 200
 set.seed(1)
 i <- as.integer(round(runif (N, 0, 100)))
-j <- as.integer(3* rpois (N, lam=15))
+j <- as.integer(3* rpois (N, lambda=15))
 x <- round(rnorm(N), 2)
 which(duplicated(cbind(i,j))) # 8 index pairs are duplicated
 
 m1 <- new("dgTMatrix", Dim = c(max(i)+1:1, max(j)+1:1), i = i, j = j, x = x)
-mc <- as(m1, "dgCMatrix")
-m2 <- as(mc, "dgTMatrix")## the same as 'm1' but without duplicates
+mc <- as(m1, "CsparseMatrix")
+m2 <- as(mc, "TsparseMatrix")## the same as 'm1' but without duplicates
 
 stopifnot(!isTRUE(all.equal.default(m1, m2)),
           all.equal(as(m1,"matrix"), as(m2,"matrix"), tolerance =1e-15),
           all.equal(crossprod(m1), crossprod(m2), tolerance =1e-15),
-          identical(mc, as(m2, "dgCMatrix")))
+          identical(mc, as(m2, "CsparseMatrix")))
 
 ### -> uniq* functions now in ../R/Auxiliaries.R
 (t2 <- system.time(um2 <- Matrix:::uniq(m1)))
 stopifnot(identical(m2,um2))
 
 ### -> error/warning condition for solve() of a singular matrix (Barry Rowlingson)
-(M <- Matrix(0+ 1:16, nc = 4))
+(M <- Matrix(0+ 1:16, ncol = 4))
 assertError(solve(M), verbose=TRUE)## ".. computationally singular" + warning + caches LU
 assertError(solve(t(M)))
 options(warn=2) # no more warnings allowed from here
